@@ -13,7 +13,7 @@ use Time::Local;
 
 our @EXPORT = qw();
 our @EXPORT_OK = qw( %subs_offset2 %subs_offset1 %subs_offset1_init );
-
+# our @ISA = qw(Set::Infinite::Function); 
 
 =head2 NAME
 
@@ -33,7 +33,7 @@ Flavio Soibelmann Glock - fglock@pucrs.br
 
 =cut
 
- 
+# our $day_size =    $Set::Infinite::Quantize_Date::day_size; 
 our $day_size =    timegm(0,0,0,2,3,2001) - timegm(0,0,0,1,3,2001);
 our $hour_size =   $day_size / 24;
 our $minute_size = $hour_size / 60;
@@ -54,50 +54,10 @@ Returned $object+$offset1, $object+$offset2 may be scalars or objects.
 =cut
 
 our %subs_offset2 = (
-	weekdays =>	sub {
-		# offsets to week-day specified
-		# 0 = first sunday from today (or today if today is sunday)
-		# 1 = first monday from today (or today if today is monday)
-		# 6 = first friday from today (or today if today is friday)
-		# 13 = second friday from today 
-		# -1 = last saturday from today (not today, even if today were saturday)
-		# -2 = last friday
-		my ($self, $index1, $index2) = @_;
-		return ($self, $self, 0) if $self == &inf;
-		# my $class = ref($self);
-		my @date = gmtime( $self ); 
-		my $wday = $date[6];
-		my ($tmp1, $tmp2);
-
-		$tmp1 = $index1 - $wday;
-		if ($index1 >= 0) { 
-			$tmp1 += 7 if $tmp1 < 0; # it will only happen next week 
-		}
-		else {
-			$tmp1 += 7 if $tmp1 < -7; # if will happen this week
-		} 
-
-		$tmp2 = $index2 - $wday;
-		if ($index2 >= 0) { 
-			$tmp2 += 7 if $tmp2 < 0; # it will only happen next week 
-		}
-		else {
-			$tmp2 += 7 if $tmp2 < -7; # if will happen this week
-		} 
-
-		# print " [ OFS:weekday $self $tmp1 $tmp2 ] \n";
-		# $date[3] += $tmp1;
-		$tmp1 = $self + $tmp1 * $day_size;
-		# $date[3] += $tmp2 - $tmp1;
-		$tmp2 = $self + $tmp2 * $day_size;
-
-		my $cmp = $tmp1 <=> $tmp2;
-		($tmp1, $tmp2, $cmp);
-	},
 	years => 	sub {
 		my ($self, $index, $index2) = @_;
 		return ($self, $self, 0) if $self == &inf;
-		# my $class = ref($self);
+		my $class = ref($self);
 		# print " [ofs:year:$self -- $index]\n";
 		my @date = gmtime( $self ); 
 		$date[5] +=	1900 + $index;
@@ -114,7 +74,7 @@ our %subs_offset2 = (
 		my ($self, $index, $index2) = @_;
 		# print " [ofs:month:$self -- $index]\n";
 		return ($self, $self, 0) if $self == &inf;
-		# my $class = ref($self);
+		my $class = ref($self);
 		my @date = gmtime( $self );
 
 		my $mon = 	$date[4] + $index; 
@@ -180,6 +140,10 @@ our %subs_offset2 = (
 	},
 );
 
+# our $day_size = timegm(0,0,0,2,3,2001) - timegm(0,0,0,1,3,2001);
+# our $hour_size = $day_size / 24;
+# our $minute_size = $hour_size / 60;
+# our $second_size = $minute_size / 60;
 
 our @week_start = ( 0, -1, -2, -3, 3, 2, 1, 0, -1, -2, -3, 3, 2, 1, 0 );
 
@@ -214,6 +178,7 @@ our %subs_offset1 = (
 		my @time = gmtime($epoch);
 		# print " [QT_D:weekyears:$self->{date_begin}[5] + $self->{quant} * $index]\n";
 		# year modulo week
+		# print " [QT:weekyears: ",$self->{date_begin}[5] + $self->{quant} * $index," epoch=$epoch ]\n";
 		# print " [QT:weekyears: time = ",join(";", @time )," ]\n";
 		$epoch += ( $week_start[$time[6] + 7 - $self->{wkst}] ) * $day_size;
 		# print " [QT:weekyears: week=",join(";", gmtime($epoch) )," wkst=$self->{wkst} tbl[",$time[6] + 7 - $self->{wkst},"]=",$week_start[$time[6] + 7 - $self->{wkst}]," ]\n\n";
